@@ -16,6 +16,7 @@ from typing import (
     Optional,
     Type,
     TypeVar,
+    Union,
 )
 
 import pysnooper
@@ -28,6 +29,7 @@ from .model.pydantic import (
     DataModelField,
     dump_resolve_reference_action,
 )
+from .model.pydantic.types import DataTypeManager
 from .parser.base import Parser
 from .version import version as __version__
 
@@ -110,9 +112,18 @@ class Error(Exception):
         return self.message
 
 
+def get_first_file(path: Path) -> Path:  # pragma: no cover
+    if path.is_file():
+        return path
+    elif path.is_dir():
+        for child in path.rglob('*'):
+            if child.is_file():
+                return child
+    raise Error('File not found')
+
+
 def generate(
-    input_name: str,
-    input_text: str,
+    input_: Union[Path, str],
     input_file_type: InputFileType,
     output: Optional[Path],
     target_python_version: PythonVersion,
@@ -128,8 +139,14 @@ def generate(
     allow_population_by_field_name: bool = False,
     use_default_on_required_field: bool = False,
 ) -> None:
+    input_text: Optional[str] = None
     if input_file_type == InputFileType.Auto:
         try:
+            input_text = (
+                input_
+                if isinstance(input_, str)
+                else get_first_file(input_).read_text()
+            )
             input_file_type = (
                 InputFileType.OpenAPI
                 if is_openapi(input_text)
@@ -149,12 +166,10 @@ def generate(
 
         if input_file_type in [InputFileType.Json, InputFileType.Yaml]:
             try:
-                if input_file_type == InputFileType.Json:
-                    obj: Dict[Any, Any] = json.loads(input_text)
-                else:
-                    import yaml
-
-                    obj = yaml.safe_load(input_text)
+                if isinstance(input_, Path) and input_.is_dir():  # pragma: no cover
+                    raise Error(f'Input must be a file for {input_file_type}')
+                input_text = input_ if isinstance(input_, str) else input_.read_text()
+                obj: Dict[Any, Any] = yaml.safe_load(input_text)
             except:
                 raise Error('Invalid file format')
             from genson import SchemaBuilder
@@ -164,14 +179,11 @@ def generate(
             input_text = json.dumps(builder.to_schema())
 
     parser = parser_class(
-        BaseModel,
-        CustomRootType,
-        DataModelField,
+        source=input_text or input_,
         base_class=base_class,
         custom_template_dir=custom_template_dir,
         extra_template_data=extra_template_data,
         target_python_version=target_python_version,
-        text=input_text,
         dump_resolve_reference_action=dump_resolve_reference_action,
         validation=validation,
         field_constraints=field_constraints,
@@ -179,33 +191,39 @@ def generate(
         strip_default_none=strip_default_none,
         aliases=aliases,
         allow_population_by_field_name=allow_population_by_field_name,
-        file_path=Path(input_name),
         use_default_on_required_field=use_default_on_required_field,
     )
 
     with chdir(output):
-        result = parser.parse()
+        results = parser.parse()
+    if isinstance(input_, str):
+        input_filename = '<stdin>'
+    else:
+        input_filename = input_.name
+    if isinstance(results, str):
 
-    if isinstance(result, str):
-        modules = {output: result}
+        modules = {output: (results, input_filename)}
     else:
         if output is None:
             raise Error('Modular references require an output directory')
         if output.suffix:
             raise Error('Modular references require an output directory, not a file')
         modules = {
-            output.joinpath(*name): body for name, body in sorted(result.items())
+            output.joinpath(*name): (result.body, str(result.source or input_filename))
+            for name, result in sorted(results.items())
         }
 
     timestamp = datetime.now(timezone.utc).replace(microsecond=0).isoformat()
+
     header = f'''\
 # generated by datamodel-codegen:
-#   filename:  {Path(input_name).name}'''
+#   filename:  {{filename}}'''
     if not disable_timestamp:
         header += f'\n#   timestamp: {timestamp}'
 
     file: Optional[IO[Any]]
-    for path, body in modules.items():
+    for path, body_and_filename in modules.items():
+        body, filename = body_and_filename
         if path is not None:
             if not path.parent.exists():
                 path.parent.mkdir(parents=True)
@@ -213,7 +231,7 @@ def generate(
         else:
             file = None
 
-        print(header, file=file)
+        print(header.format(filename=filename), file=file)
         if body:
             print('', file=file)
             print(body.rstrip(), file=file)
