@@ -2,40 +2,20 @@ import keyword
 import re
 from abc import ABC, abstractmethod
 from collections import defaultdict
-from functools import wraps
 from pathlib import Path
-from typing import Any, Callable, DefaultDict, Dict, List, Optional, Set
+from typing import Any, DefaultDict, Dict, List, Optional, Set
 
 from jinja2 import Environment, FileSystemLoader, Template
 from pydantic import BaseModel, root_validator
 
-from datamodel_code_generator.imports import (
-    IMPORT_LIST,
-    IMPORT_OPTIONAL,
-    IMPORT_UNION,
-    Import,
-)
-from datamodel_code_generator.types import DataType, Types
+from datamodel_code_generator.imports import IMPORT_OPTIONAL, Import
+from datamodel_code_generator.types import DataType
 
 TEMPLATE_DIR: Path = Path(__file__).parents[0] / 'template'
 
-UNION: str = 'Union'
 OPTIONAL: str = 'Optional'
-LIST: str = 'List'
 
-
-def optional(func: Callable[..., Any]) -> Callable[..., Any]:
-    @wraps(func)
-    def inner(self: 'DataModelFieldBase', *args: Any, **kwargs: Any) -> Optional[str]:
-        type_hint: Optional[str] = func(self, *args, **kwargs)
-        if self.required:
-            return type_hint
-        self.imports.append(IMPORT_OPTIONAL)
-        if type_hint is None or type_hint == '':
-            return OPTIONAL
-        return f'{OPTIONAL}[{type_hint}]'
-
-    return inner
+ALL_MODEL: str = '#all#'
 
 
 class ConstraintsBase(BaseModel):
@@ -51,40 +31,34 @@ class DataModelFieldBase(BaseModel):
     examples: Any = None
     description: Optional[str]
     title: Optional[str]
-    data_types: List[DataType] = []
-    is_list: bool = False
-    is_union: bool = False
-    imports: List[Import] = []
-    type_hint: Optional[str] = None
-    unresolved_types: List[str] = []
+    data_type: DataType
     constraints: Any = None
     strip_default_none: bool = False
 
+    @property
+    def type_hint(self) -> str:
+        type_hint = self.data_type.type_hint
+        if self.required:
+            return type_hint
+        if type_hint is None or type_hint == '':
+            return OPTIONAL
+        return f'{OPTIONAL}[{type_hint}]'
+
+    @property
+    def imports(self) -> List[Import]:
+        if not self.required:
+            return self.data_type.imports_ + [IMPORT_OPTIONAL]
+        return self.data_type.imports_
+
+    @property
+    def unresolved_types(self) -> Set[str]:
+        return self.data_type.unresolved_types
+
     @property
     def field(self) -> Optional[str]:
         """for backwards compatibility"""
         return None
 
-    @optional
-    def _get_type_hint(self) -> Optional[str]:
-        type_hint = ', '.join(d.type_hint for d in self.data_types)
-        if not type_hint:
-            if self.is_list:
-                self.imports.append(IMPORT_LIST)
-                return LIST
-            return ''
-        if len(self.data_types) == 1:
-            if self.is_list:
-                self.imports.append(IMPORT_LIST)
-                return f'{LIST}[{type_hint}]'
-            return type_hint
-        if self.is_list:
-            self.imports.append(IMPORT_LIST)
-            self.imports.append(IMPORT_UNION)
-            return f'{LIST}[{UNION}[{type_hint}]]'
-        self.imports.append(IMPORT_UNION)
-        return f'{UNION}[{type_hint}]'
-
     @property
     def method(self) -> Optional[str]:
         return None
@@ -106,14 +80,6 @@ class DataModelFieldBase(BaseModel):
             values['name'] = name
         return values
 
-    def __init__(self, **values: Any) -> None:
-        super().__init__(**values)
-        for data_type in self.data_types:
-            self.unresolved_types.extend(data_type.unresolved_types)
-            if data_type.imports_:
-                self.imports.extend(data_type.imports_)
-        self.type_hint = self._get_type_hint()
-
     @property
     def represented_default(self) -> str:
         return repr(self.default)
@@ -164,6 +130,7 @@ class DataModel(TemplateBase, ABC):
         auto_import: bool = True,
         reference_classes: Optional[List[str]] = None,
         methods: Optional[List[str]] = None,
+        path: Optional[Path] = None,
     ) -> None:
         if not self.TEMPLATE_FILE_PATH:
             raise Exception('TEMPLATE_FILE_PATH is undefined')
@@ -181,12 +148,13 @@ class DataModel(TemplateBase, ABC):
         self.base_class: Optional[str] = None
         base_classes = [base_class for base_class in base_classes or [] if base_class]
         self.base_classes: List[str] = base_classes
+        self.path: Optional[Path] = path
 
-        self.reference_classes: List[str] = [
+        self.reference_classes: Set[str] = {
             r for r in base_classes if r != self.BASE_CLASS
-        ] if base_classes else []
+        } if base_classes else set()
         if reference_classes:
-            self.reference_classes.extend(reference_classes)
+            self.reference_classes.update(reference_classes)
 
         if self.base_classes:
             self.base_class = ', '.join(self.base_classes)
@@ -212,12 +180,16 @@ class DataModel(TemplateBase, ABC):
             if extra_template_data is not None
             else defaultdict(dict)
         )
+        if extra_template_data:
+            all_model_extra_template_data = extra_template_data.get(ALL_MODEL)
+            if all_model_extra_template_data:
+                self.extra_template_data.update(all_model_extra_template_data)
 
-        unresolved_types: Set[str] = set()
+        unresolved_types: Set[str] = {*()}
         for field in self.fields:
-            unresolved_types.update(set(field.unresolved_types))
+            unresolved_types.update(field.unresolved_types)
 
-        self.reference_classes = list(set(self.reference_classes) | unresolved_types)
+        self.reference_classes |= unresolved_types
 
         if auto_import:
             for field in self.fields:
@@ -227,6 +199,16 @@ class DataModel(TemplateBase, ABC):
 
         super().__init__(template_file_path=template_file_path)
 
+    @property
+    def module_path(self) -> List[str]:
+        if self.path:
+            return [
+                *self.path.parts[:-1],
+                self.path.stem,
+                *self.name.split('.')[:-1],
+            ]
+        return self.name.split('.')[:-1]
+
     def render(self) -> str:
         response = self._render(
             class_name=self.class_name,
@@ -237,8 +219,3 @@ class DataModel(TemplateBase, ABC):
             **self.extra_template_data,
         )
         return response
-
-    @classmethod
-    @abstractmethod
-    def get_data_type(cls, types: Types, **kwargs: Any) -> DataType:
-        raise NotImplementedError
