@@ -8,6 +8,7 @@ from typing import (
     Callable,
     DefaultDict,
     Dict,
+    Iterator,
     List,
     Mapping,
     Optional,
@@ -17,38 +18,21 @@ from typing import (
     Union,
 )
 
-import inflect
 from pydantic import BaseModel
 
 from datamodel_code_generator.format import format_code
 
 from ..format import PythonVersion
 from ..imports import IMPORT_ANNOTATIONS, Import, Imports
-from ..model.base import ALL_MODEL, DataModel, DataModelFieldBase, TemplateBase
-from ..types import DataType, DataTypePy36
-
-inflect_engine = inflect.engine()
+from ..model import pydantic as pydantic_model
+from ..model.base import ALL_MODEL, DataModel, DataModelFieldBase
+from ..reference import ModelResolver
+from ..types import DataType, DataTypeManager
 
 _UNDER_SCORE_1 = re.compile(r'(.)([A-Z][a-z]+)')
 _UNDER_SCORE_2 = re.compile('([a-z0-9])([A-Z])')
 
 
-def get_singular_name(name: str, suffix: str = 'Item') -> str:
-    singular_name = inflect_engine.singular_noun(name)
-    if singular_name is False:
-        singular_name = f'{name}{suffix}'
-    return singular_name
-
-
-def snake_to_upper_camel(word: str) -> str:
-    prefix = ''
-    if word.startswith('_'):
-        prefix = '_'
-        word = word[1:]
-
-    return prefix + ''.join(x[0].upper() + x[1:] for x in word.split('_') if x)
-
-
 def camel_to_snake(string: str) -> str:
     subbed = _UNDER_SCORE_1.sub(r'\1_\2', string)
     return _UNDER_SCORE_2.sub(r'\1_\2', subbed).lower()
@@ -67,10 +51,7 @@ def set_strip_default_none(field: DataModelFieldBase) -> None:
     field.strip_default_none = True
 
 
-def dump_templates(templates: Union[DataModel, List[DataModel]]) -> str:
-    if isinstance(templates, TemplateBase):
-        templates = [templates]
-
+def dump_templates(templates: List[DataModel]) -> str:
     return '\n\n\n'.join(str(m) for m in templates)
 
 
@@ -98,7 +79,7 @@ def sort_data_models(
             sorted_data_models[model.name] = model
             require_update_action_models.append(model.name)
         elif (
-            not set(model.reference_classes) - {model.name} - set(sorted_data_models)
+            not model.reference_classes - {model.name} - set(sorted_data_models)
         ):  # reference classes have been resolved
             sorted_data_models[model.name] = model
             if model.name in model.reference_classes:
@@ -148,141 +129,33 @@ def relative(current_module: str, reference: str) -> Tuple[str, str]:
     return left, right
 
 
-class Reference(BaseModel):
-    path: List[str]
-    original_name: str
-    name: str
-    loaded: bool = True
+class Result(BaseModel):
+    body: str
+    source: Optional[Path]
 
 
-class ModelResolver:
-    def __init__(self, aliases: Optional[Mapping[str, str]] = None) -> None:
-        self.references: Dict[str, Reference] = {}
-        self.aliases = {**aliases} if aliases is not None else {}
+class Source(BaseModel):
+    path: Path
+    text: str
 
-    @staticmethod
-    def _get_path(path: List[str]) -> str:
-        if '#' in path:  # remote
-            delimiter = path.index('#')
-            return f"{''.join(path[:delimiter])}#/{''.join(path[delimiter + 1:])}"
-        return '/'.join(path)
-
-    def add_ref(self, ref: str) -> Reference:
-        if ref in self.references:
-            return self.references[ref]
-        split_ref = ref.rsplit('/', 1)
-        if len(split_ref) == 1:
-            # TODO Support $id with $ref
-            # https://json-schema.org/understanding-json-schema/structuring.html#using-id-with-ref
-            raise NotImplementedError('It is not support to use $id with $ref')
-        parents, original_name = split_ref  # type: str, str
-        loaded: bool = not ref.startswith(('https://', 'http://'))
-        if not original_name:
-            original_name = Path(parents).stem
-            loaded = False
-        name = self.get_class_name(original_name, unique=False)
-        reference = Reference(
-            path=ref.split('/'), original_name=original_name, name=name, loaded=loaded,
-        )
-        self.references[ref] = reference
-        return reference
-
-    def add(
-        self,
-        path: List[str],
-        original_name: str,
-        *,
-        class_name: bool = False,
-        singular_name: bool = False,
-        unique: bool = False,
-        singular_name_suffix: str = 'Item',
-    ) -> Reference:
-        joined_path: str = self._get_path(path)
-        if joined_path in self.references:
-            return self.references[joined_path]
-        if not original_name:
-            original_name = Path(joined_path.split('#')[0]).stem
-        if class_name:
-            name = self.get_class_name(original_name, unique)
-            if singular_name:  # pragma: no cover
-                name = get_singular_name(name, singular_name_suffix)
-        elif singular_name:
-            name = get_singular_name(original_name, singular_name_suffix)
-            if unique:  # pragma: no cover
-                name = self._get_uniq_name(name)
-        elif unique:
-            name = self._get_uniq_name(original_name)
-        else:
-            name = original_name
-        reference = Reference(path=path, original_name=original_name, name=name)
-        self.references[joined_path] = reference
-        return reference
-
-    def get(
-        self, path: Union[List[str], str]
-    ) -> Optional[Reference]:  # pragma: no cover
-        if isinstance(path, str):
-            return self.references.get(path)
-        return self.references[self._get_path(path)]
-
-    def get_class_name(self, field_name: str, unique: bool = True) -> str:
-        if '.' in field_name:
-            split_name = [self.get_valid_name(n) for n in field_name.split('.')]
-            prefix, field_name = '.'.join(split_name[:-1]), split_name[-1]
-            prefix += '.'
-        else:
-            prefix = ''
-
-        field_name = self.get_valid_name(field_name)
-        upper_camel_name = snake_to_upper_camel(field_name)
-        if unique:
-            class_name = self._get_uniq_name(upper_camel_name, camel=True)
-        else:
-            class_name = upper_camel_name
-
-        return f'{prefix}{class_name}'
-
-    def _get_uniq_name(self, name: str, camel: bool = False) -> str:
-        uniq_name: str = name
-        count: int = 1
-        while uniq_name in [r.name for r in self.references.values()]:
-            if camel:
-                uniq_name = f'{name}{count}'
-            else:
-                uniq_name = f'{name}_{count}'
-            count += 1
-        return uniq_name
-
-    def get_valid_name(self, name: str, camel: bool = False) -> str:
-        # TODO: when first character is a number
-        replaced_name = re.sub(r'\W', '_', name)
-        if re.match(r'^[0-9]', replaced_name):
-            replaced_name = f'field_{replaced_name}'
-        # if replaced_name.isidentifier() and not iskeyword(replaced_name):
-        # return self.get_uniq_name(replaced_name, camel)
-        return replaced_name
-
-    def get_valid_field_name_and_alias(
-        self, field_name: str
-    ) -> Tuple[str, Optional[str]]:
-        if field_name in self.aliases:
-            return self.aliases[field_name], field_name
-        valid_name = self.get_valid_name(field_name)
-        return valid_name, None if field_name == valid_name else field_name
+    @classmethod
+    def from_path(cls, path: Path, base_path: Path) -> 'Source':
+        return cls(path=path.relative_to(base_path), text=path.read_text(),)
 
 
 class Parser(ABC):
     def __init__(
         self,
-        data_model_type: Type[DataModel],
-        data_model_root_type: Type[DataModel],
-        data_model_field_type: Type[DataModelFieldBase] = DataModelFieldBase,
+        source: Union[str, Path, List[Path]],
+        *,
+        data_model_type: Type[DataModel] = pydantic_model.BaseModel,
+        data_model_root_type: Type[DataModel] = pydantic_model.CustomRootType,
+        data_type_manager_type: Type[DataTypeManager] = pydantic_model.DataTypeManager,
+        data_model_field_type: Type[DataModelFieldBase] = pydantic_model.DataModelField,
         base_class: Optional[str] = None,
         custom_template_dir: Optional[Path] = None,
         extra_template_data: Optional[DefaultDict[str, Dict[str, Any]]] = None,
         target_python_version: PythonVersion = PythonVersion.PY_37,
-        text: Optional[str] = None,
-        result: Optional[List[DataModel]] = None,
         dump_resolve_reference_action: Optional[Callable[[List[str]], str]] = None,
         validation: bool = False,
         field_constraints: bool = False,
@@ -290,18 +163,18 @@ class Parser(ABC):
         strip_default_none: bool = False,
         aliases: Optional[Mapping[str, str]] = None,
         allow_population_by_field_name: bool = False,
-        file_path: Optional[Path] = None,
         use_default_on_required_field: bool = False,
     ):
-
+        self.data_type_manager: DataTypeManager = data_type_manager_type(
+            target_python_version
+        )
         self.data_model_type: Type[DataModel] = data_model_type
         self.data_model_root_type: Type[DataModel] = data_model_root_type
         self.data_model_field_type: Type[DataModelFieldBase] = data_model_field_type
         self.imports: Imports = Imports()
         self.base_class: Optional[str] = base_class
         self.target_python_version: PythonVersion = target_python_version
-        self.text: Optional[str] = text
-        self.results: List[DataModel] = result or []
+        self.results: List[DataModel] = []
         self.dump_resolve_reference_action: Optional[
             Callable[[List[str]], str]
         ] = dump_resolve_reference_action
@@ -311,16 +184,16 @@ class Parser(ABC):
         self.strip_default_none: bool = strip_default_none
         self.use_default_on_required_field: bool = use_default_on_required_field
 
-        if self.target_python_version == PythonVersion.PY_36:
-            self.data_type: Type[DataType] = DataTypePy36
-        else:
-            self.data_type = DataType
+        self.current_source_path: Optional[Path] = None
 
-        if file_path:
-            self.base_path: Path = file_path.absolute().parent
+        if isinstance(source, Path):
+            self.base_path = (
+                source.absolute() if source.is_dir() else source.absolute().parent
+            )
         else:
             self.base_path = Path.cwd()
 
+        self.source: Union[str, Path, List[Path]] = source
         self.custom_template_dir = custom_template_dir
         self.extra_template_data: DefaultDict[
             str, Any
@@ -336,19 +209,38 @@ class Parser(ABC):
         if self.strip_default_none:
             self.field_preprocessors.append(set_strip_default_none)
 
+    @property
+    def iter_source(self) -> Iterator[Source]:
+        if isinstance(self.source, str):
+            yield Source(path=Path(), text=self.source)
+        elif isinstance(self.source, Path):  # pragma: no cover
+            if self.source.is_dir():
+                for path in self.source.rglob('*'):
+                    if path.is_file():
+                        yield Source.from_path(path, self.base_path)
+            else:
+                yield Source.from_path(self.source, self.base_path)
+        elif isinstance(self.source, list):  # pragma: no cover
+            for path in self.source:
+                yield Source.from_path(path, self.base_path)
+
     def append_result(self, data_model: DataModel) -> None:
         for field_preprocessor in self.field_preprocessors:
             for field in data_model.fields:
                 field_preprocessor(field)
         self.results.append(data_model)
 
+    @property
+    def data_type(self) -> Type[DataType]:
+        return self.data_type_manager.data_type
+
     @abstractmethod
     def parse_raw(self) -> None:
         raise NotImplementedError
 
     def parse(
         self, with_import: Optional[bool] = True, format_: Optional[bool] = True
-    ) -> Union[str, Dict[Tuple[str, ...], str]]:
+    ) -> Union[str, Dict[Tuple[str, ...], Result]]:
 
         self.parse_raw()
 
@@ -360,23 +252,25 @@ class Parser(ABC):
             self.results
         )
 
-        results: Dict[Tuple[str, ...], str] = {}
+        results: Dict[Tuple[str, ...], Result] = {}
 
-        module_key = lambda x: (*x.name.split('.')[:-1],)
+        module_key = lambda x: x.module_path
 
         # process in reverse order to correctly establish module levels
         grouped_models = groupby(
             sorted(sorted_data_models.values(), key=module_key, reverse=True),
             key=module_key,
         )
-        for module, models in ((k, [*v]) for k, v in grouped_models):
+        for module, models in (
+            (k, [*v]) for k, v in grouped_models
+        ):  # type: Tuple[str, ...], List[DataModel]
             module_path = '.'.join(module)
 
             init = False
             if module:
                 parent = (*module[:-1], '__init__.py')
                 if parent not in results:
-                    results[parent] = ''
+                    results[parent] = Result(body='')
                 if (*module, '__init__.py') in results:
                     module = (*module, '__init__.py')
                     init = True
@@ -395,14 +289,29 @@ class Parser(ABC):
                     models_to_update += [model.name]
                 imports.append(model.imports)
                 for field in model.fields:
-                    type_hint = field.type_hint
-                    if type_hint is None:  # pragma: no cover
-                        continue
-                    for data_type in field.data_types:
-                        if '.' not in data_type.type:
+                    for data_type in field.data_type.all_data_types:  # type: DataType
+                        if not data_type.type or (
+                            '.' not in data_type.type and data_type.module_name is None
+                        ):
                             continue
-                        from_, import_ = relative(module_path, data_type.type)
+                        type_ = (
+                            f"{data_type.module_name}.{data_type.type}"
+                            if data_type.module_name
+                            else data_type.type
+                        )
+                        from_, import_ = relative(module_path, type_)
                         full_path = f'{from_}/{import_}'
+                        name = type_.rsplit('.', 1)[-1]
+                        if data_type.reference:
+                            reference = self.model_resolver.get(
+                                data_type.reference.path
+                            )
+                            if (
+                                reference
+                                and reference.actual_module_name == module_path
+                            ):
+                                model.reference_classes.remove(name)
+                                continue
                         if full_path in alias_map:
                             alias = alias_map[full_path] or import_
                         else:
@@ -410,14 +319,11 @@ class Parser(ABC):
                                 full_path.split('/'), import_, unique=True
                             ).name
                             alias_map[full_path] = None if alias == import_ else alias
-                        name = data_type.type.rsplit('.', 1)[-1]
-                        pattern = re.compile(rf'\b{re.escape(data_type.type)}\b')
-                        if from_ and import_:
-                            type_hint = pattern.sub(rf'{alias}.{name}', type_hint)
-                        else:
-                            type_hint = pattern.sub(name, type_hint)
-
-                    field.type_hint = type_hint
+                        new_name = f'{alias}.{name}' if from_ and import_ else name
+                        if name in model.reference_classes:
+                            model.reference_classes.remove(name)
+                            model.reference_classes.add(new_name)
+                        data_type.type = new_name
 
                 for ref_name in model.reference_classes:
                     from_, import_ = relative(module_path, ref_name)
@@ -445,10 +351,10 @@ class Parser(ABC):
             if format_:
                 body = format_code(body, self.target_python_version)
 
-            results[module] = body
+            results[module] = Result(body=body, source=models[0].path)
 
         # retain existing behaviour
         if [*results] == [('__init__.py',)]:
-            return results[('__init__.py',)]
+            return results[('__init__.py',)].body
 
         return results
