@@ -8,7 +8,6 @@ from typing import (
     Mapping,
     Optional,
     Set,
-    Tuple,
     Type,
     Union,
 )
@@ -18,24 +17,19 @@ from pydantic import BaseModel, Field, root_validator, validator
 
 from datamodel_code_generator import snooper_to_methods
 from datamodel_code_generator.format import PythonVersion
-from datamodel_code_generator.imports import (
-    IMPORT_ANY,
-    IMPORT_DICT,
-    IMPORT_LIST,
-    IMPORT_OPTIONAL,
-)
 from datamodel_code_generator.model import DataModel, DataModelFieldBase
 from datamodel_code_generator.model.enum import Enum
 
+from ..model import pydantic as pydantic_model
 from ..parser.base import Parser
-from ..types import DataType, Types
+from ..types import DataType, DataTypeManager, Types
 
 
 def get_model_by_path(schema: Dict[str, Any], keys: List[str]) -> Dict[str, Any]:
-    if len(keys) == 0:
+    if not keys:
         return schema
     elif len(keys) == 1:
-        return schema[keys[0]]
+        return schema.get(keys[0], {})
     return get_model_by_path(schema[keys[0]], keys[1:])
 
 
@@ -78,6 +72,19 @@ json_schema_data_formats: Dict[str, Dict[str, Types]] = {
 
 
 class JsonSchemaObject(BaseModel):
+    __constraint_fields__: Set[str] = {
+        'exclusiveMinimum',
+        'minimum',
+        'exclusiveMaximum',
+        'maximum',
+        'multipleOf',
+        'minItems',
+        'maxItems',
+        'minLength',
+        'maxLength',
+        'pattern',
+    }
+
     @root_validator(pre=True)
     def validate_exclusive_maximum_and_exclusive_minimum(
         cls, values: Dict[str, Any]
@@ -106,6 +113,8 @@ class JsonSchemaObject(BaseModel):
     maxLength: Optional[int]
     minimum: Optional[float]
     maximum: Optional[float]
+    minItems: Optional[int]
+    maxItems: Optional[int]
     multipleOf: Optional[float]
     exclusiveMaximum: Union[float, bool, None]
     exclusiveMinimum: Union[float, bool, None]
@@ -148,6 +157,10 @@ class JsonSchemaObject(BaseModel):
     def has_default(self) -> bool:
         return 'default' in self.__fields_set__
 
+    @property
+    def has_constraint(self) -> bool:
+        return bool(self.__constraint_fields__ & self.__fields_set__)
+
 
 JsonSchemaObject.update_forward_refs()
 
@@ -156,15 +169,16 @@ JsonSchemaObject.update_forward_refs()
 class JsonSchemaParser(Parser):
     def __init__(
         self,
-        data_model_type: Type[DataModel],
-        data_model_root_type: Type[DataModel],
-        data_model_field_type: Type[DataModelFieldBase] = DataModelFieldBase,
+        source: Union[str, Path, List[Path]],
+        *,
+        data_model_type: Type[DataModel] = pydantic_model.BaseModel,
+        data_model_root_type: Type[DataModel] = pydantic_model.CustomRootType,
+        data_type_manager_type: Type[DataTypeManager] = pydantic_model.DataTypeManager,
+        data_model_field_type: Type[DataModelFieldBase] = pydantic_model.DataModelField,
         base_class: Optional[str] = None,
         custom_template_dir: Optional[Path] = None,
         extra_template_data: Optional[DefaultDict[str, Dict[str, Any]]] = None,
         target_python_version: PythonVersion = PythonVersion.PY_37,
-        text: Optional[str] = None,
-        result: Optional[List[DataModel]] = None,
         dump_resolve_reference_action: Optional[Callable[[List[str]], str]] = None,
         validation: bool = False,
         field_constraints: bool = False,
@@ -172,64 +186,52 @@ class JsonSchemaParser(Parser):
         strip_default_none: bool = False,
         aliases: Optional[Mapping[str, str]] = None,
         allow_population_by_field_name: bool = False,
-        file_path: Optional[Path] = None,
         use_default_on_required_field: bool = False,
     ):
         super().__init__(
-            data_model_type,
-            data_model_root_type,
-            data_model_field_type,
-            base_class,
-            custom_template_dir,
-            extra_template_data,
-            target_python_version,
-            text,
-            result,
-            dump_resolve_reference_action,
-            validation,
-            field_constraints,
-            snake_case_field,
-            strip_default_none,
-            aliases,
-            allow_population_by_field_name,
-            file_path,
-            use_default_on_required_field,
+            source=source,
+            data_model_type=data_model_type,
+            data_model_root_type=data_model_root_type,
+            data_type_manager_type=data_type_manager_type,
+            data_model_field_type=data_model_field_type,
+            base_class=base_class,
+            custom_template_dir=custom_template_dir,
+            extra_template_data=extra_template_data,
+            target_python_version=target_python_version,
+            dump_resolve_reference_action=dump_resolve_reference_action,
+            validation=validation,
+            field_constraints=field_constraints,
+            snake_case_field=snake_case_field,
+            strip_default_none=strip_default_none,
+            aliases=aliases,
+            allow_population_by_field_name=allow_population_by_field_name,
+            use_default_on_required_field=use_default_on_required_field,
         )
 
         self.remote_object_cache: Dict[str, Dict[str, Any]] = {}
 
-    def get_data_type(self, obj: JsonSchemaObject) -> List[DataType]:
+    def get_data_type(self, obj: JsonSchemaObject) -> DataType:
         if obj.type is None:
-            return [
-                self.data_type(
-                    type='Any', version_compatible=True, imports_=[IMPORT_ANY]
-                )
-            ]
-        if isinstance(obj.type, list):
-            types: List[str] = [t for t in obj.type if t != 'null']
-            format_ = 'default'
-            if len(types) == 1 and 'null' in obj.type:
-                type_ = self.data_model_type.get_data_type(
-                    json_schema_data_formats[types[0]][format_],
-                    **obj.dict() if not self.field_constraints else {},
-                )
-                type_.optional = True
-                type_.imports_ = (
-                    [IMPORT_OPTIONAL]
-                    if type_.imports_ is None
-                    else [*type_.imports_, IMPORT_OPTIONAL]
-                )
-                return [type_]
-        else:
-            types = [obj.type]
-            format_ = obj.format or 'default'
-        return [
-            self.data_model_type.get_data_type(
-                json_schema_data_formats[t][format_],
+            return self.data_type_manager.get_data_type(Types.any)
+
+        def _get_data_type(type_: str, format__: str) -> DataType:
+            return self.data_type_manager.get_data_type(
+                json_schema_data_formats[type_][format__],
                 **obj.dict() if not self.field_constraints else {},
             )
-            for t in types
-        ]
+
+        if isinstance(obj.type, list):
+            return self.data_type(
+                data_types=[
+                    _get_data_type(t, 'default') for t in obj.type if t != 'null'
+                ],
+                is_optional='null' in obj.type,
+            )
+        return _get_data_type(obj.type, obj.format or 'default')
+
+    def get_ref_data_type(self, ref: str) -> DataType:
+        reference = self.model_resolver.add_ref(ref)
+        return self.data_type.from_reference(reference)
 
     def set_additional_properties(self, name: str, obj: JsonSchemaObject) -> None:
         if not obj.additionalProperties:
@@ -248,75 +250,53 @@ class JsonSchemaParser(Parser):
 
     def parse_list_item(
         self, name: str, target_items: List[JsonSchemaObject], path: List[str]
-    ) -> List[DataType]:
+    ) -> DataType:
         data_types: List[DataType] = []
         for item in target_items:
             if item.ref:  # $ref
-                data_types.append(
-                    self.data_type(
-                        type=self.model_resolver.add_ref(item.ref).name,
-                        ref=True,
-                        version_compatible=True,
-                    )
-                )
+                data_types.append(self.get_ref_data_type(item.ref))
             elif not any(v for k, v in vars(item).items() if k != 'type'):
                 # trivial types
-                data_types.extend(self.get_data_type(item))
+                data_types.append(self.get_data_type(item))
             elif (
                 item.is_array
                 and isinstance(item.items, JsonSchemaObject)
                 and not any(v for k, v in vars(item.items).items() if k != 'type')
             ):
                 # trivial item types
-                types = [t.type_hint for t in self.get_data_type(item.items)]
                 data_types.append(
                     self.data_type(
-                        type=f"List[Union[{', '.join(types)}]]"
-                        if len(types) > 1
-                        else f"List[{types[0]}]",
-                        imports_=[IMPORT_LIST],
+                        data_types=[self.get_data_type(item.items)], is_list=True,
                     )
                 )
             else:
                 data_types.append(
-                    self.data_type(
-                        type=self.parse_object(
-                            name, item, path, singular_name=True
-                        ).name,
-                        ref=True,
-                        version_compatible=True,
+                    self.data_type.from_model_name(
+                        self.parse_object(name, item, path, singular_name=True).name,
                     )
                 )
-        return data_types
+        return self.data_type(data_types=data_types)
 
     def parse_any_of(
         self, name: str, obj: JsonSchemaObject, path: List[str]
-    ) -> List[DataType]:
+    ) -> DataType:
         return self.parse_list_item(name, obj.anyOf, path)
 
     def parse_one_of(
         self, name: str, obj: JsonSchemaObject, path: List[str]
-    ) -> List[DataType]:
+    ) -> DataType:
         return self.parse_list_item(name, obj.oneOf, path)
 
     def parse_all_of(
         self, name: str, obj: JsonSchemaObject, path: List[str]
-    ) -> List[DataType]:
+    ) -> DataType:
         fields: List[DataModelFieldBase] = []
         base_classes: List[DataType] = []
         for all_of_item in obj.allOf:
             if all_of_item.ref:  # $ref
-                base_classes.append(
-                    self.data_type(
-                        type=self.model_resolver.add_ref(all_of_item.ref).name,
-                        ref=True,
-                        version_compatible=True,
-                    )
-                )
-
+                base_classes.append(self.get_ref_data_type(all_of_item.ref))
             else:
-                fields_ = self.parse_object_fields(all_of_item, path)
-                fields.extend(fields_)
+                fields.extend(self.parse_object_fields(all_of_item, path))
         class_name = self.model_resolver.add(
             path, name, class_name=True, unique=True
         ).name
@@ -324,18 +304,19 @@ class JsonSchemaParser(Parser):
         data_model_type = self.data_model_type(
             class_name,
             fields=fields,
-            base_classes=[b.type for b in base_classes],
+            base_classes=[b.type_hint for b in base_classes],
             auto_import=False,
             custom_base_class=self.base_class,
             custom_template_dir=self.custom_template_dir,
             extra_template_data=self.extra_template_data,
+            path=self.current_source_path,
         )
         # add imports for the fields
         for f in fields:
             data_model_type.imports.extend(f.imports)
         self.append_result(data_model_type)
 
-        return [self.data_type(type=class_name, ref=True, version_compatible=True)]
+        return self.data_type.from_model_name(class_name)
 
     def parse_object_fields(
         self, obj: JsonSchemaObject, path: List[str]
@@ -347,46 +328,34 @@ class JsonSchemaParser(Parser):
         fields: List[DataModelFieldBase] = []
 
         for field_name, field in properties.items():
-            is_list: bool = False
-            is_union: bool = False
-            field_types: List[DataType]
+            field_type: DataType
             original_field_name: str = field_name
             constraints: Optional[Mapping[str, Any]] = None
             field_name, alias = self.model_resolver.get_valid_field_name_and_alias(
                 field_name
             )
             if field.ref:
-                field_types = [
-                    self.data_type(
-                        type=self.model_resolver.add_ref(field.ref).name,
-                        ref=True,
-                        version_compatible=True,
-                    )
-                ]
+                field_type = self.get_ref_data_type(field.ref)
             elif field.is_array:
-                array_field, array_field_classes = self.parse_array_fields(
+                array_field = self.parse_array_fields(
                     field_name, field, [*path, field_name]
                 )
-                field_types = array_field.data_types
-                is_list = True
-                is_union = True
+                field_type = array_field.data_type
+                constraints = field.dict()
             elif field.anyOf:
-                field_types = self.parse_any_of(field_name, field, [*path, field_name])
+                field_type = self.parse_any_of(field_name, field, [*path, field_name])
             elif field.oneOf:
-                field_types = self.parse_one_of(field_name, field, [*path, field_name])
+                field_type = self.parse_one_of(field_name, field, [*path, field_name])
             elif field.allOf:
-                field_types = self.parse_all_of(field_name, field, [*path, field_name])
+                field_type = self.parse_all_of(field_name, field, [*path, field_name])
             elif field.is_object:
                 if field.properties:
-                    field_types = [
-                        self.data_type(
-                            type=self.parse_object(
-                                field_name, field, [*path, field_name], unique=True
-                            ).name,
-                            ref=True,
-                            version_compatible=True,
-                        )
-                    ]
+                    field_type = self.data_type.from_model_name(
+                        self.parse_object(
+                            field_name, field, [*path, field_name], unique=True
+                        ).name
+                    )
+
                 elif isinstance(field.additionalProperties, JsonSchemaObject):
                     unresolved_type: str
                     field_class_name = self.model_resolver.add(
@@ -404,12 +373,9 @@ class JsonSchemaParser(Parser):
                             unresolved_type = self.model_resolver.add_ref(
                                 field.additionalProperties.items.ref
                             ).name
-                            additional_properties_type = self.data_type(
-                                type=f"List[{unresolved_type}]",
-                                imports_=[IMPORT_LIST],
-                                ref=True,
-                                version_compatible=True,
-                            ).type
+                            additional_properties_type = self.data_type.from_model_name(
+                                unresolved_type, is_list=True
+                            ).type_hint
                         else:
                             additional_properties_type = (
                                 unresolved_type
@@ -431,28 +397,21 @@ class JsonSchemaParser(Parser):
                             else field,
                         ).name
 
-                    field_types = [
-                        self.data_type(
-                            type=f'Dict[str, {additional_properties_type}]',
-                            imports_=[IMPORT_DICT],
-                            unresolved_types=[unresolved_type],
-                        )
-                    ]
+                    field_type = self.data_type(
+                        type=additional_properties_type,
+                        is_dict=True,
+                        unresolved_types={unresolved_type},
+                    )
+
                 else:
-                    field_types = [
-                        self.data_type(
-                            type='Dict[str, Any]', imports_=[IMPORT_ANY, IMPORT_DICT],
-                        )
-                    ]
+                    field_type = self.data_type_manager.get_data_type(Types.object)
             elif field.enum:
                 enum = self.parse_enum(
                     field_name, field, [*path, field_name], unique=True
                 )
-                field_types = [
-                    self.data_type(type=enum.name, ref=True, version_compatible=True)
-                ]
+                field_type = self.data_type.from_model_name(enum.name)
             else:
-                field_types = self.get_data_type(field)
+                field_type = self.get_data_type(field)
                 if self.field_constraints:
                     constraints = field.dict()
             if self.use_default_on_required_field and field.has_default:
@@ -467,10 +426,8 @@ class JsonSchemaParser(Parser):
                     description=field.description,
                     default=field.default,
                     title=field.title,
-                    data_types=field_types,
+                    data_type=field_type,
                     required=required,
-                    is_list=is_list,
-                    is_union=is_union,
                     alias=alias,
                     constraints=constraints,
                 )
@@ -498,43 +455,45 @@ class JsonSchemaParser(Parser):
             custom_base_class=self.base_class,
             custom_template_dir=self.custom_template_dir,
             extra_template_data=self.extra_template_data,
+            path=self.current_source_path,
         )
         self.append_result(data_model_type)
         return data_model_type
 
     def parse_array_fields(
         self, name: str, obj: JsonSchemaObject, path: List[str]
-    ) -> Tuple[DataModelFieldBase, List[DataType]]:
+    ) -> DataModelFieldBase:
         if isinstance(obj.items, JsonSchemaObject):
             items = [obj.items]
         else:
             items = obj.items or []
         item_obj_data_types: List[DataType] = []
-        is_union: bool = False
-        for item in items:
-            if item.ref:
+        for index, item in enumerate(items):
+            if item.has_constraint:
+                model = self.model_resolver.add(
+                    [*path, f'items{index}'],
+                    name,
+                    class_name=True,
+                    singular_name=True,
+                    unique=True,
+                )
                 item_obj_data_types.append(
-                    self.data_type(
-                        type=self.model_resolver.add_ref(item.ref).name,
-                        ref=True,
-                        version_compatible=True,
+                    self.data_type.from_model_name(
+                        self.parse_root_type(model.name, item, path,).name
                     )
                 )
+            elif item.ref:
+                item_obj_data_types.append(self.get_ref_data_type(item.ref))
             elif isinstance(item, JsonSchemaObject) and item.properties:
                 item_obj_data_types.append(
-                    self.data_type(
-                        type=self.parse_object(
-                            name, item, path, singular_name=True
-                        ).name,
-                        ref=True,
-                        version_compatible=True,
+                    self.data_type.from_model_name(
+                        self.parse_object(name, item, path, singular_name=True).name,
                     )
                 )
             elif item.anyOf:
-                item_obj_data_types.extend(self.parse_any_of(name, item, path))
-                is_union = True
+                item_obj_data_types.append(self.parse_any_of(name, item, path))
             elif item.allOf:
-                item_obj_data_types.extend(
+                item_obj_data_types.append(
                     self.parse_all_of(
                         self.model_resolver.add(path, name, singular_name=True).name,
                         item,
@@ -543,23 +502,21 @@ class JsonSchemaParser(Parser):
                 )
             elif item.enum:
                 item_obj_data_types.append(
-                    self.data_type(
-                        type=self.parse_enum(name, item, path, singular_name=True).name,
-                        ref=True,
-                        version_compatible=True,
+                    self.data_type.from_model_name(
+                        self.parse_enum(name, item, path, singular_name=True).name,
                     )
                 )
             elif item.is_array:
-                array_field, array_field_classes = self.parse_array_fields(
+                array_field = self.parse_array_fields(
                     self.model_resolver.add(path, name, class_name=True).name,
                     item,
                     path,
                 )
-                item_obj_data_types.extend(array_field.data_types)
+                item_obj_data_types.append(array_field.data_type)
             else:
-                item_obj_data_types.extend(self.get_data_type(item))
+                item_obj_data_types.append(self.get_data_type(item))
         field = self.data_model_field_type(
-            data_types=item_obj_data_types,
+            data_type=self.data_type(data_types=item_obj_data_types, is_list=True,),
             example=obj.example,
             examples=obj.examples,
             default=obj.default,
@@ -567,15 +524,14 @@ class JsonSchemaParser(Parser):
             title=obj.title,
             required=not obj.nullable
             and not (obj.has_default and self.use_default_on_required_field),
-            is_list=True,
-            is_union=is_union,
+            constraints=obj.dict(),
         )
-        return field, item_obj_data_types
+        return field
 
     def parse_array(
         self, name: str, obj: JsonSchemaObject, path: List[str]
     ) -> DataModel:
-        field, item_obj_names = self.parse_array_fields(name, obj, [*path, name])
+        field = self.parse_array_fields(name, obj, [*path, name])
         self.model_resolver.add(path, name)
         data_model_root = self.data_model_root_type(
             name,
@@ -590,43 +546,37 @@ class JsonSchemaParser(Parser):
 
     def parse_root_type(
         self, name: str, obj: JsonSchemaObject, path: List[str]
-    ) -> None:
+    ) -> DataModel:
         if obj.type:
-            types: List[DataType] = self.get_data_type(obj)
+            data_type: DataType = self.get_data_type(obj)
         elif obj.anyOf:
-            types = self.parse_any_of(name, obj, [*path, name])
+            data_type = self.parse_any_of(name, obj, [*path, name])
         elif obj.ref:
-            types = [
-                self.data_type(
-                    type=self.model_resolver.add_ref(obj.ref).name,
-                    ref=True,
-                    version_compatible=True,
-                )
-            ]
+            data_type = self.get_ref_data_type(obj.ref)
         else:
-            types = [
-                self.data_type(
-                    type='Any', version_compatible=True, imports_=[IMPORT_ANY]
-                )
-            ]
+            data_type = self.data_type_manager.get_data_type(Types.any)
+
         data_model_root_type = self.data_model_root_type(
             name,
             [
                 self.data_model_field_type(
-                    data_types=types,
+                    data_type=data_type,
                     description=obj.description,
                     example=obj.example,
                     examples=obj.examples,
                     default=obj.default,
                     required=not obj.nullable
                     and not (obj.has_default and self.use_default_on_required_field),
+                    constraints=obj.dict() if self.field_constraints else {},
                 )
             ],
             custom_base_class=self.base_class,
             custom_template_dir=self.custom_template_dir,
             extra_template_data=self.extra_template_data,
+            path=self.current_source_path,
         )
         self.append_result(data_model_root_type)
+        return data_model_root_type
 
     def parse_enum(
         self,
@@ -651,7 +601,12 @@ class JsonSchemaParser(Parser):
                 else:
                     field_name = f'{obj.type}_{enum_part}'
             enum_fields.append(
-                self.data_model_field_type(name=field_name, default=default)
+                self.data_model_field_type(
+                    name=field_name,
+                    default=default,
+                    data_type=self.data_type_manager.get_data_type(Types.any),
+                    required=True,
+                )
             )
         enum_name = self.model_resolver.add(
             path,
@@ -712,10 +667,13 @@ class JsonSchemaParser(Parser):
                     else:
                         models = ref_body
                         model_name = Path(object_path).stem
-                    self.parse_raw_obj(
-                        model_name, models, [str(full_path), '#', *object_paths],
-                    )
-                    self.model_resolver.add_ref(obj.ref).loaded = True
+                    if models:  # pragma: no cover
+                        self.parse_raw_obj(
+                            model_name, models, [relative_path, '#', *object_paths],
+                        )
+                    self.model_resolver.add_ref(
+                        obj.ref, actual_module_name=''
+                    ).loaded = True
 
         if obj.items:
             if isinstance(obj.items, JsonSchemaObject):
@@ -750,10 +708,19 @@ class JsonSchemaParser(Parser):
         self.parse_ref(obj, path)
 
     def parse_raw(self) -> None:
-        raw_obj: Dict[Any, Any] = yaml.safe_load(self.text)  # type: ignore
-        obj_name = raw_obj.get('title', 'Model')
-        obj_name = self.model_resolver.add([], obj_name, unique=False).name
-        self.parse_raw_obj(obj_name, raw_obj, [])
-        definitions = raw_obj.get('definitions', {})
-        for key, model in definitions.items():
-            self.parse_raw_obj(key, model, ['#/definitions', key])
+        if isinstance(self.source, list) or (
+            isinstance(self.source, Path) and self.source.is_dir()
+        ):
+            self.current_source_path = Path()
+        for source in self.iter_source:
+            if self.current_source_path is not None:
+                self.current_source_path = source.path
+            path_parts = list(source.path.parts)
+            self.model_resolver.set_current_root(path_parts)
+            raw_obj: Dict[Any, Any] = yaml.safe_load(source.text)
+            obj_name = raw_obj.get('title', 'Model')
+            obj_name = self.model_resolver.add(path_parts, obj_name, unique=False).name
+            self.parse_raw_obj(obj_name, raw_obj, path_parts)
+            definitions = raw_obj.get('definitions', {})
+            for key, model in definitions.items():
+                self.parse_raw_obj(key, model, [*path_parts, '#/definitions', key])
