@@ -6,8 +6,9 @@ from unittest.mock import Mock
 import pytest
 import yaml
 
-from datamodel_code_generator import DataModelField
+from datamodel_code_generator import DataModelField, DataTypeManager
 from datamodel_code_generator.imports import IMPORT_OPTIONAL, Import
+from datamodel_code_generator.model import DataModelFieldBase
 from datamodel_code_generator.model.pydantic import BaseModel, CustomRootType
 from datamodel_code_generator.parser.base import dump_templates
 from datamodel_code_generator.parser.jsonschema import (
@@ -19,6 +20,10 @@ from datamodel_code_generator.types import DataType
 
 DATA_PATH: Path = Path(__file__).parents[1] / 'data' / 'jsonschema'
 
+EXPECTED_JSONSCHEMA_PATH = (
+    Path(__file__).parents[1] / 'data' / 'expected' / 'parser' / 'jsonschema'
+)
+
 
 @pytest.mark.parametrize(
     'schema,path,model',
@@ -34,36 +39,32 @@ def test_get_model_by_path(schema: Dict, path: str, model: Dict):
     assert get_model_by_path(schema, path.split('/') if path else []) == model
 
 
-def test_json_schema_parser_parse_ref():
-    parser = JsonSchemaParser(
-        BaseModel, CustomRootType, data_model_field_type=DataModelField
-    )
-    parser.parse_raw_obj = Mock()
-    external_parent_path = Path(DATA_PATH / 'external_parent.json')
-    parser.base_path = external_parent_path.parent
-    parser.excludes_ref_path = set()
-    external_parent = external_parent_path.read_text()
-    obj = JsonSchemaObject.parse_raw(external_parent)
-
-    parser.parse_ref(obj, [])
-    # parser.parse_raw_obj.assert_has_calls(
-    #     [
-    #         call(
-    #             'Yaml',
-    #             {'properties': {'firstName': {'type': 'string'}}, 'type': 'object'},
-    #         ),
-    #         call(
-    #             'Json',
-    #             {'properties': {'firstName': {'type': 'string'}}, 'type': 'object'},
-    #         ),
-    #     ]
-    # )
+# def test_json_schema_parser_parse_ref():
+#     parser = JsonSchemaParser()
+#     parser.parse_raw_obj = Mock()
+#     external_parent_path = Path(DATA_PATH / 'external_parent.json')
+#     parser.base_path = external_parent_path.parent
+#     parser.excludes_ref_path = set()
+#     external_parent = external_parent_path.read_text()
+#     obj = JsonSchemaObject.parse_raw(external_parent)
+#
+#     parser.parse_ref(obj, [])
+#     # parser.parse_raw_obj.assert_has_calls(
+#     #     [
+#     #         call(
+#     #             'Yaml',
+#     #             {'properties': {'firstName': {'type': 'string'}}, 'type': 'object'},
+#     #         ),
+#     #         call(
+#     #             'Json',
+#     #             {'properties': {'firstName': {'type': 'string'}}, 'type': 'object'},
+#     #         ),
+#     #     ]
+#     # )
 
 
 def test_json_schema_object_ref_url_json(mocker):
-    parser = JsonSchemaParser(
-        BaseModel, CustomRootType, data_model_field_type=DataModelField
-    )
+    parser = JsonSchemaParser('')
     obj = JsonSchemaObject.parse_obj(
         {'$ref': 'https://example.org/schema.json#/definitions/User'}
     )
@@ -89,9 +90,7 @@ def test_json_schema_object_ref_url_json(mocker):
 
 
 def test_json_schema_object_ref_url_yaml(mocker):
-    parser = JsonSchemaParser(
-        BaseModel, CustomRootType, data_model_field_type=DataModelField
-    )
+    parser = JsonSchemaParser('')
     obj = JsonSchemaObject.parse_obj(
         {'$ref': 'https://example.org/schema.yaml#/definitions/User'}
     )
@@ -111,9 +110,7 @@ def test_json_schema_object_ref_url_yaml(mocker):
 
 
 def test_json_schema_object_cached_ref_url_yaml(mocker):
-    parser = JsonSchemaParser(
-        BaseModel, CustomRootType, data_model_field_type=DataModelField
-    )
+    parser = JsonSchemaParser('')
 
     obj = JsonSchemaObject.parse_obj(
         {
@@ -143,9 +140,7 @@ class User(BaseModel):
 
 
 def test_json_schema_ref_url_json(mocker):
-    parser = JsonSchemaParser(
-        BaseModel, CustomRootType, data_model_field_type=DataModelField
-    )
+    parser = JsonSchemaParser('')
     obj = {
         "type": "object",
         "properties": {
@@ -230,7 +225,7 @@ class User(BaseModel):
     ],
 )
 def test_parse_object(source_obj, generated_classes):
-    parser = JsonSchemaParser(BaseModel, CustomRootType)
+    parser = JsonSchemaParser(data_model_field_type=DataModelFieldBase, source='')
     parser.parse_object('Person', JsonSchemaObject.parse_obj(source_obj), [])
     assert dump_templates(list(parser.results)) == generated_classes
 
@@ -246,14 +241,12 @@ def test_parse_object(source_obj, generated_classes):
                 "description": "This field accepts any object",
             },
             """class AnyObject(BaseModel):
-    __root__: Any""",
+    __root__: Any = Field(..., description='This field accepts any object')""",
         )
     ],
 )
 def test_parse_any_root_object(source_obj, generated_classes):
-    parser = JsonSchemaParser(
-        BaseModel, CustomRootType, data_model_field_type=DataModelField
-    )
+    parser = JsonSchemaParser('')
     parser.parse_root_type('AnyObject', JsonSchemaObject.parse_obj(source_obj), [])
     assert dump_templates(list(parser.results)) == generated_classes
 
@@ -268,9 +261,7 @@ def test_parse_any_root_object(source_obj, generated_classes):
     ],
 )
 def test_parse_one_of_object(source_obj, generated_classes):
-    parser = JsonSchemaParser(
-        BaseModel, CustomRootType, data_model_field_type=DataModelField
-    )
+    parser = JsonSchemaParser('')
     parser.parse_raw_obj('onOfObject', source_obj, [])
     assert dump_templates(list(parser.results)) == generated_classes
 
@@ -314,19 +305,14 @@ def test_parse_one_of_object(source_obj, generated_classes):
     ],
 )
 def test_parse_default(source_obj, generated_classes):
-    parser = JsonSchemaParser(
-        BaseModel, CustomRootType, data_model_field_type=DataModelField
-    )
+    parser = JsonSchemaParser('')
     parser.parse_raw_obj('Defaults', source_obj, [])
     assert dump_templates(list(parser.results)) == generated_classes
 
 
 def test_parse_nested_array():
     parser = JsonSchemaParser(
-        BaseModel,
-        CustomRootType,
-        data_model_field_type=DataModelField,
-        text=(DATA_PATH / 'nested_array.json').read_text(),
+        DATA_PATH / 'nested_array.json', data_model_field_type=DataModelFieldBase,
     )
     parser.parse()
     assert (
@@ -367,12 +353,12 @@ def test_get_data_type(schema_type, schema_format, result_type, from_, import_):
     if from_ and import_:
         imports_: Optional[List[Import]] = [Import(from_=from_, import_=import_)]
     else:
-        imports_ = None
+        imports_ = []
 
-    parser = JsonSchemaParser(BaseModel, CustomRootType)
+    parser = JsonSchemaParser('')
     assert parser.get_data_type(
         JsonSchemaObject(type=schema_type, format=schema_format)
-    ) == [DataType(type=result_type, imports_=imports_)]
+    ) == DataType(type=result_type, imports_=imports_)
 
 
 @pytest.mark.parametrize(
@@ -380,12 +366,9 @@ def test_get_data_type(schema_type, schema_format, result_type, from_, import_):
     [(['integer', 'number'], ['int', 'float']), (['integer', 'null'], ['int']),],
 )
 def test_get_data_type_array(schema_types, result_types):
-    parser = JsonSchemaParser(BaseModel, CustomRootType)
-    assert parser.get_data_type(JsonSchemaObject(type=schema_types)) == [
-        DataType(
-            type=r,
-            optional='null' in schema_types,
-            imports_=[IMPORT_OPTIONAL] if 'null' in schema_types else None,
-        )
-        for r in result_types
-    ]
+    parser = JsonSchemaParser('')
+    assert parser.get_data_type(JsonSchemaObject(type=schema_types)) == DataType(
+        data_types=[DataType(type=r,) for r in result_types],
+        is_optional='null' in schema_types,
+        imports_=[IMPORT_OPTIONAL] if 'null' in schema_types else [],
+    )
